import { Analyzer } from '@previewjs/analyzer-api';
import { TypeAnalyzer } from '@previewjs/type-analyzer';
import { Reader } from '@previewjs/vfs';
import { Logger } from 'pino';
import vite from 'vite';
import * as express from 'express';
import { RPC, RequestOf, ResponseOf } from '@previewjs/api';

type PackageDependencies = Record<string, {
    /** @deprecated */
    majorVersion: number;
    readInstalledVersion(): Promise<string | null>;
}>;

interface FrameworkPluginFactory {
    isCompatible(dependencies: PackageDependencies): Promise<boolean>;
    create(options: {
        rootDir: string;
        reader: Reader;
        logger: Logger;
        dependencies: PackageDependencies;
    }): Promise<FrameworkPlugin>;
}
interface FrameworkPlugin extends Analyzer {
    readonly pluginApiVersion?: number;
    readonly name: string;
    readonly defaultWrapperPath: string;
    readonly previewDirPath: string;
    readonly viteConfig: (configuredPlugins: vite.Plugin[]) => vite.UserConfig;
    dispose(): void;
}

type RegisterRPC = <E extends RPC<any, any>>(endpoint: E, handler: RequestHandler<RequestOf<E>, ResponseOf<E>>) => void;
type RequestHandler<Req, Res> = (request: Req) => Promise<Res>;

type OnServerStart = (options: {
    versionCode?: string;
    registerRPC: RegisterRPC;
    workspace: Workspace;
}) => Promise<{
    middlewares?: express.RequestHandler[];
}>;

declare class NoCompatiblePluginError extends Error {
    constructor(message: string);
}
declare function createWorkspace({ rootDir, frameworkPlugins, logger, reader, onServerStart, }: {
    rootDir: string;
    frameworkPlugins: FrameworkPluginFactory[];
    logger?: Logger;
    reader?: Reader;
    onServerStart?: OnServerStart;
}): Promise<Workspace>;
/**
 * Returns the absolute directory path of the closest ancestor containing package.json.
 */
declare function findWorkspaceRoot(absoluteFilePath: string): string | null;
interface Workspace {
    rootDir: string;
    reader: Reader;
    frameworkPluginName: string;
    typeAnalyzer: Omit<TypeAnalyzer, "dispose">;
    crawlFiles: Analyzer["crawlFiles"];
    startServer: (options?: {
        port?: number;
    }) => Promise<PreviewServer>;
    dispose(): Promise<void>;
}
interface PreviewServer {
    url(): string;
    stop(): Promise<void>;
}

export { type FrameworkPlugin, type FrameworkPluginFactory, NoCompatiblePluginError, type OnServerStart, type PackageDependencies, type PreviewServer, type Workspace, createWorkspace, findWorkspaceRoot };
